@@ -80,6 +80,7 @@ struct _DataFormPriv
 
 	/* EDIT and BROWSE modes only */
         gulong           numrow;
+	GtkWidget       *top_notebook;
 
 	/* EDIT, INSERT and BROWSE modes only */
 	GtkWidget       *actions_area;
@@ -89,7 +90,6 @@ struct _DataFormPriv
 struct _ValueRef {
         gpointer        ref;       /* DbField or QueryParamValue */
         EntryEntity    *entry;
-	GdaValue       *orig_value;
 };
 #define VALUE_REF(vr) ((ValueRef*)(vr))
 
@@ -222,6 +222,7 @@ data_form_init (DataForm * form)
 	form->priv->entries = NULL;
 	form->priv->master = NULL;
 	form->priv->children = NULL;
+	form->priv->top_notebook = NULL;
 	form->priv->internal_query = NULL;
 	form->priv->internal_rs = NULL;
 	form->priv->numrow = 0;
@@ -370,8 +371,6 @@ data_form_finalize (GObject   * object)
 		if (df->priv->value_refs) {
 			list = df->priv->value_refs;
 			while (list) {
-				if (VALUE_REF (list->data)->orig_value)
-					gda_value_free (VALUE_REF (list->data)->orig_value);
 				g_free (list->data);
 				list = g_slist_next (list);
 			}
@@ -436,34 +435,50 @@ static void build_edit_insert_form_structure (DataForm * form);
 static void 
 data_form_edit_initialize (DataForm * form)
 {
-	GtkWidget *topwid = NULL;
+	GtkWidget *topwid, *wid, *buttons;
 
+	/* notebook to show either "No Data" or a form if there are some data */
+	topwid = gtk_notebook_new ();
+	gtk_notebook_set_show_tabs (GTK_NOTEBOOK (topwid), FALSE);
+
+	/* First page, "No Data" */
+	wid = gtk_label_new (_("No Data!"));
+	gtk_notebook_append_page (GTK_NOTEBOOK (topwid), wid, NULL);
+	gtk_widget_show (wid);
+
+
+	/* Second Page, the form itself */
+	build_edit_insert_form_structure (form);
+	if (0) {
+		/* FIXME: update when glade forms are introduced */
+		/* we have a custom glade form here */
+		wid = gtk_label_new ("Glade imported form to create");
+		gtk_widget_show (wid);
+	}
+	else {
+		/* default builtin form */
+		wid = build_complete_noglade_form_widget (form);
+		gtk_widget_show (wid);
+	}
+	gtk_notebook_append_page (GTK_NOTEBOOK (topwid), wid, NULL);
+	form->priv->top_notebook = topwid;
+
+
+	/* Action buttons, etc */
+	gtk_widget_show (topwid);
+	gtk_box_pack_start_defaults (GTK_BOX (form), topwid);
+	
+	buttons = data_form_add_action_buttons (form, G_CALLBACK (data_form_user_action_cb), form);
+	gtk_widget_show_all (buttons);
+	gtk_box_pack_start (GTK_BOX (form), buttons, FALSE, FALSE, GNOME_PAD/2.);
+
+
+	/* Default shown page */
 	if (! query_exec_get_data_set (form->priv->qx) || 
 	    (server_resultset_get_nbtuples (query_exec_get_data_set (form->priv->qx)) == 0))
-		topwid = gtk_label_new (_("No Data!"));
-	else {
-		build_edit_insert_form_structure (form);
-
-		if (0) {
-			/* FIXME: update when glade forms are introduced */
-			/* we have a custom glade form here */
-			topwid = gtk_label_new ("Glade imported form to create");
-		}
-		else {
-			/* default builtin form */
-			topwid = build_complete_noglade_form_widget (form);
-		}
-	}
-
-	if (topwid) {
-		GtkWidget *buttons;
-		gtk_widget_show_all (topwid);
-		gtk_box_pack_start_defaults (GTK_BOX (form), topwid);
-		
-		buttons = data_form_add_action_buttons (form, G_CALLBACK (data_form_user_action_cb), form);
-		gtk_widget_show_all (buttons);
-		gtk_box_pack_start (GTK_BOX (form), buttons, FALSE, FALSE, GNOME_PAD/2.);
-	}
+		gtk_notebook_set_current_page (GTK_NOTEBOOK (topwid), 0);
+	else 
+		gtk_notebook_set_current_page (GTK_NOTEBOOK (topwid), 1);
 }
 
 static void 
@@ -504,7 +519,6 @@ build_edit_insert_form_structure (DataForm * form)
 					DataForm *df;
 					ValueRef *vr;
 					EntryEntity *ee;
-					const GdaValue *value;
 					gint pos;
 
 					/* set the other QueryView */
@@ -551,17 +565,6 @@ build_edit_insert_form_structure (DataForm * form)
 						vr = g_new0 (ValueRef, 1);
 						vr->ref = ofield;
 						vr->entry = ee;
-						if (form->priv->mode == DATA_FORM_EDIT_MODE) {
-							value = server_resultset_get_gdavalue (query_exec_get_data_set (form->priv->qx),
-											       pos,
-											       form->priv->numrow);
-
-							vr->orig_value = gda_value_copy (value);
-						}
-						else
-							vr->orig_value = gda_value_new_null ();
-
-						vr->entry = ee;
 							
 						form->priv->value_refs = g_slist_append (form->priv->value_refs,
 											 vr);
@@ -571,7 +574,7 @@ build_edit_insert_form_structure (DataForm * form)
 				else { /* No QueryJoin for that QueryField */
 					if (qf->is_printed && !qf->is_hidden) {
 						DataEntry *de;
-						DataDisplayFns *fns;
+						DataHandler *dh;
 						const GdaValue *value;
 						DbField *field = NULL;
 						EntryEntity *ee;
@@ -590,34 +593,31 @@ build_edit_insert_form_structure (DataForm * form)
 						srv = query_exec_get_query_ref (form->priv->qx)->conf->srv;
 						field = query_field_field_get_db_field (qf);
 						if (server_access_is_object_bound (srv, G_OBJECT (qf)))
-							fns = server_access_get_object_display_fns (srv, 
-												    G_OBJECT (qf));
+							dh = server_access_get_object_handler (srv, 
+											       G_OBJECT (qf));
 						else
-							fns = server_access_get_object_display_fns (srv, 
-												    G_OBJECT (field));
+							dh = server_access_get_object_handler (srv, 
+											       G_OBJECT (field));
 						if (value) 
-							de = DATA_ENTRY ((fns->widget_from_value) (value));
+							de = DATA_ENTRY (data_handler_get_widget_from_value (dh, value));
 						else {
 							GdaValue *myvalue;
 							myvalue = gda_value_new_null ();
-							de = DATA_ENTRY ((fns->widget_from_value) (myvalue));
+							de = DATA_ENTRY (data_handler_get_widget_from_value (dh, myvalue));
 							gda_value_free (myvalue);
 						}
-						de->fns = fns;
-						de->query_field = qf;
+						data_entry_set_query_field (de, qf);
 						
 						if (form->priv->mode == DATA_FORM_INSERT_MODE) {
-							if (field->default_val) {
-								de->value_is_defaultval = TRUE;
-								de->is_default_possible = TRUE;
-							}
+							if (field->default_value) 
+								data_entry_set_is_default_set (de, TRUE);
 							else {
-								de->value_is_defaultval = FALSE;
-								de->is_default_possible = FALSE;
+								data_entry_set_is_default_set (de, FALSE);
+								data_entry_set_is_default_allowed (de, FALSE);
 							}
 						}
 						if (! field->null_allowed)
-							de->value_required = TRUE;
+							data_entry_set_is_null_allowed (de, FALSE);
 
 						data_entry_refresh_display (de);
 						
@@ -637,14 +637,6 @@ build_edit_insert_form_structure (DataForm * form)
 							vr->ref = field;
 							vr->entry = ee;
 							pos = query_get_pos_by_field (query_exec_get_query_ref (form->priv->qx), qf);
-							if (form->priv->mode == DATA_FORM_EDIT_MODE) {
-								value = server_resultset_get_gdavalue (query_exec_get_data_set (form->priv->qx),
-												       form->priv->numrow, pos);
-								vr->orig_value = gda_value_copy (value);
-							}
-							else 
-								vr->orig_value = gda_value_new_null ();
-
 							vr->entry = ee;
 								
 							form->priv->value_refs = g_slist_append (form->priv->value_refs,
@@ -683,16 +675,18 @@ static void data_form_insert_initialize (DataForm * form)
 		/* FIXME: update when glade forms are introduced */
 		/* we have a custom glade form here */
 		topwid = gtk_label_new ("Glade imported form to create");
+		gtk_widget_show (topwid);
 	}
 	else {
 		/* default builtin form */
 		topwid = build_complete_noglade_form_widget (form);
+		gtk_widget_show (topwid);
 	}
 
 
 	if (topwid) {
 		GtkWidget *buttons;
-		gtk_widget_show_all (topwid);
+		gtk_widget_show (topwid);
 		gtk_box_pack_start_defaults (GTK_BOX (form), topwid);
 		
 		buttons = data_form_add_action_buttons (form, G_CALLBACK (data_form_user_action_cb), form);
@@ -749,11 +743,12 @@ static void data_form_select_initialize (DataForm * form)
 		else {
 			/* default builtin form */
 			topwid = build_complete_noglade_form_widget (form);
+			gtk_widget_show (topwid);
 		}
 	}
 
 	if (topwid) {
-		gtk_widget_show_all (topwid);
+		gtk_widget_show (topwid);
 		gtk_box_pack_start_defaults (GTK_BOX (form), topwid);
 	}
 }
@@ -792,29 +787,38 @@ build_complete_noglade_form_widget (DataForm * form)
 	
 	list = form->priv->entries;
 	while (list) {
-		GtkWidget *wid=NULL, *label;
-		
+		GtkWidget *wid = NULL, *label;
+		gchar *str;
+
 		if (ENTRY_ENTITY (list->data)->entry) {
 			wid = GTK_WIDGET (ENTRY_ENTITY (list->data)->entry);
+			gtk_widget_show (wid);
 
 			/* fetch the "contents_modified" signal */
 			g_signal_connect (wid, "contents_modified",
 					  G_CALLBACK (data_entry_contents_modified_cb), form);
 		}
-		if (ENTRY_ENTITY (list->data)->form)
+		if (ENTRY_ENTITY (list->data)->form) {
 			wid = GTK_WIDGET (ENTRY_ENTITY (list->data)->form);
+			gtk_widget_show (wid);
+		}
+
 		gtk_table_attach_defaults (GTK_TABLE (topwid), wid,
 					   1, 2, i, i+1);
 		
 		if (ENTRY_ENTITY (list->data)->qf->name)
-			label = gtk_label_new (ENTRY_ENTITY (list->data)->qf->name);
+			str = g_strdup_printf ("%s:", ENTRY_ENTITY (list->data)->qf->name);
 		else
-			label = gtk_label_new (DB_FIELD (ENTRY_ENTITY (list->data)->value_ref->ref)->name);
+			str = g_strdup_printf ("%s:", DB_FIELD (ENTRY_ENTITY (list->data)->value_ref->ref)->name);
+
+		label = gtk_label_new (str);
+		gtk_widget_show (label);
+		g_free (str);
 		
 		gtk_table_attach (GTK_TABLE (topwid), label,
 				  0, 1, i, i+1, 
-				  0, 0, GNOME_PAD/2., 0);
-		
+				  GTK_FILL, 0, GNOME_PAD/2., 0);
+		gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
 		
 		list = g_slist_next (list);
 		i++;
@@ -848,8 +852,16 @@ data_entry_refresh_actions_area (DataForm *form)
 	if (!collector)
 		return;
 
-	is_first = data_form_is_on_first_entry (form);
-	is_last = data_form_is_on_last_entry (form);
+	if ((form->priv->mode == DATA_FORM_EDIT_MODE) &&
+	    (server_resultset_get_nbtuples (query_exec_get_data_set (form->priv->qx)) == 0)) {
+		is_first = TRUE;
+		is_last = TRUE;
+	}
+	else {
+		is_first = data_form_is_on_first_entry (form);
+		is_last = data_form_is_on_last_entry (form);
+	}
+
 	is_modified = data_form_is_diff_than_orig (form);
 
 	/* make sure there is no missing value in the form */
@@ -860,7 +872,9 @@ data_entry_refresh_actions_area (DataForm *form)
 		while (list && to_commit) {
 			if (ENTRY_ENTITY (list->data)->entry) {
 				DataEntry *de = DATA_ENTRY (ENTRY_ENTITY (list->data)->entry);
-				if (de->value_required && data_entry_is_value_null (de))
+				if (!data_entry_get_is_null_allowed (de) && 
+				    data_entry_is_value_null (de) && 
+				    !data_entry_is_value_default (de))
 					to_commit = FALSE;
 			}
 
@@ -868,6 +882,14 @@ data_entry_refresh_actions_area (DataForm *form)
 		}
 	}
 
+	if ((form->priv->mode == DATA_FORM_EDIT_MODE) &&
+	    (server_resultset_get_nbtuples (query_exec_get_data_set (form->priv->qx)) == 0)) {
+		to_commit = FALSE;
+		is_modified = TRUE;
+	}
+
+
+
 	/* QUERY_ACTION_FIRST */
 	str = g_strdup_printf ("w%d", QUERY_ACTION_FIRST);
 	wid = g_object_get_data (G_OBJECT (collector), str);
@@ -966,7 +988,7 @@ data_form_is_diff_than_orig (DataForm *form)
 		ee = ENTRY_ENTITY (list->data);
 
 		if (ee->entry) {
-			if (DATA_ENTRY (ee->entry)->value_is_modified)
+			if (data_entry_is_value_modified (DATA_ENTRY (ee->entry)))
 				retval = TRUE;
 		}
 		
@@ -986,30 +1008,48 @@ void
 data_form_refresh (DataForm *form)
 {
 	GSList *list;
+	ServerResultset *rs;
 
 	g_return_if_fail (form && IS_DATA_FORM (form));
+	if ((form->priv->mode != DATA_FORM_EDIT_MODE) &&
+	    (form->priv->mode != DATA_FORM_BROWSE_MODE))
+		return;
 
-	list = form->priv->entries;
-	while (list) {
-		EntryEntity *ee;
+	rs = query_exec_get_data_set (form->priv->qx);
 
-		ee = ENTRY_ENTITY (list->data);
+	if (!rs || (rs && (server_resultset_get_nbtuples (rs) == 0))) {
+		/* Show the "No Data" Page */
+		gtk_notebook_set_current_page (GTK_NOTEBOOK (form->priv->top_notebook), 0);
+		data_entry_refresh_actions_area (form);
+	}
+	else {
+		/* Show the "actual Form" page */
+		gtk_notebook_set_current_page (GTK_NOTEBOOK (form->priv->top_notebook), 1);
 
-		if (ee->entry) {
-			gint pos;
-			const GdaValue *value;
+		if (form->priv->numrow > server_resultset_get_nbtuples (rs) - 1)
+			form->priv->numrow = server_resultset_get_nbtuples (rs) - 1;
 
-			pos = query_get_pos_by_field (query_exec_get_query_ref (form->priv->qx), ee->qf);
-			value = server_resultset_get_gdavalue (query_exec_get_data_set (form->priv->qx),
-							       form->priv->numrow, pos);
-			(*ee->entry->fns->widget_update)(GTK_WIDGET (ee->entry), value, TRUE);
-		}
+		list = form->priv->entries;
+		while (list) {
+			EntryEntity *ee;
+			
+			ee = ENTRY_ENTITY (list->data);
+			
+			if (ee->entry) {
+				gint pos;
+				const GdaValue *value;
+				
+				pos = query_get_pos_by_field (query_exec_get_query_ref (form->priv->qx), ee->qf);
+				value = server_resultset_get_gdavalue (rs, form->priv->numrow, pos);
+				data_entry_update_with_value (ee->entry, value, TRUE);
+			}
+			
+			if (ee->form) { /* refresh the sub form */
+				data_form_refresh (ee->form);
+			}
 		
-		if (ee->form) { /* refresh the sub form */
-			data_form_refresh (ee->form);
+			list = g_slist_next (list);
 		}
-		
-		list = g_slist_next (list);
 	}
 }
 
@@ -1060,6 +1100,7 @@ data_form_find_value_ref (DataForm *df, gpointer ref)
 
 
 static void form_do_commit (DataForm *form);
+static void form_do_delete (DataForm *form);
 static void 
 data_form_user_action_cb (GtkWidget *wid, DataForm * form)
 {
@@ -1099,7 +1140,7 @@ data_form_user_action_cb (GtkWidget *wid, DataForm * form)
 		form_do_commit (form);
 		break;
 	case QUERY_ACTION_DELETE:
-		g_print ("To be implemented in %s line %d\n", __FILE__, __LINE__);
+		form_do_delete (form);
 		break;
 	case QUERY_ACTION_EDIT:
 		g_print ("To be implemented in %s line %d\n", __FILE__, __LINE__);
@@ -1109,11 +1150,17 @@ data_form_user_action_cb (GtkWidget *wid, DataForm * form)
 	}
 }
 
+
+static gchar * make_update_sql_query (DataForm *form, GSList *param_values);
+static gchar * make_insert_sql_query (DataForm *form, GSList *param_values);
 static void 
 form_do_commit (DataForm *form)
 {
 	GSList *list, *values = NULL;
+	gchar *query;
+	ServerResultset *rs;
 
+	/* build the new values list */
 	list = form->priv->entries;
 	while (list) {
 		QueryParamValue *value;
@@ -1124,8 +1171,27 @@ form_do_commit (DataForm *form)
 			value = g_new0 (QueryParamValue, 1);
 			value->field = ENTRY_ENTITY (list->data)->qf;
 			value->value = data_entry_get_gdavalue (de);
+			value->dh = data_entry_get_data_handler (de);
+
+			/* QF type test */
+			if (value->field->field_type != QUERY_FIELD_FIELD)
+				g_warning ("EntryEntity %p does not reference a DbField\n", list->data);
+			
+			if (data_entry_is_value_default (de)) {
+				/* fetch the default GdaValue in the DbField */
+				DbField *field = query_field_field_get_db_field (value->field);
+				if (field->default_value) 
+					value->value = gda_value_copy (field->default_value);
+			}
+			else {
+				/* value from the DataEntry widget (if the value is NULL, 
+				   then the function will return a GdaValue for NULL) */
+				value->value = data_entry_get_gdavalue (de);
+			}
+				
 		}
 		else {
+			/* FIXME : data handler: get the default one for the right data type */
 			DataForm *subform = ENTRY_ENTITY (list->data)->form;
 			value = g_new0 (QueryParamValue, 1);
 			value->field = ENTRY_ENTITY (list->data)->qf;
@@ -1136,11 +1202,288 @@ form_do_commit (DataForm *form)
 		list = g_slist_next (list);
 	}
 
+	/* build the SQL statement */
+	switch  (form->priv->mode) {
+	case DATA_FORM_EDIT_MODE:
+		query = make_update_sql_query (form, values);
+		break;
+	case DATA_FORM_INSERT_MODE:
+		query = make_insert_sql_query (form, values);
+		break;
+	default:
+		query = NULL;
+		g_warning ("form_do_commit () called on the wrong mode: %d\n", form->priv->mode);
+	}
 
-	/* FIXME: about default value and NULL values */
-	g_print ("Started implementation in %s line %d\n", __FILE__, __LINE__);
+	/* free all the values */
+	list = values;
+	while (list) {
+		gda_value_free (QUERY_PARAM_VALUE (list->data)->value);
+		g_free (list->data);
+		list = g_slist_next (list);
+	}
+	g_slist_free (values);
+
+	/* Query execution */
+	rs = server_access_do_query (form->priv->qv->query->conf->srv, query, 
+				     SERVER_ACCESS_QUERY_SQL);
+	if (rs) {
+		g_signal_emit (G_OBJECT (form), data_form_signals [ACTION_REFRESH], 0);
+		
+		if (form->priv->mode == DATA_FORM_INSERT_MODE) {
+			/* close the form */
+			gtk_widget_destroy (GTK_WIDGET (form));
+		}
+		
+		g_object_unref (G_OBJECT (rs));
+	}
+	
+	g_free (query);
 }
 
+static gchar * make_where_cond_sql_part (DataForm *form);
+static gchar *
+make_update_sql_query (DataForm *form, GSList *param_values)
+{
+	GString *string;
+	gchar *str;
+	DbTable *modif_table;
+	GSList *list;
+	gboolean first_field;
+
+	g_return_val_if_fail (form->priv->mode == DATA_FORM_EDIT_MODE, NULL);
+	string = g_string_new ("UPDATE ");
+
+	modif_table = query_exec_get_modif_table (form->priv->qx);
+	g_string_append_printf (string, "%s SET ", modif_table->name);
+
+	/* New values */
+	first_field = TRUE;
+	list = param_values;
+	while (list) {
+		QueryParamValue *value;
+		DbField *field;
+		
+		value = QUERY_PARAM_VALUE (list->data);
+		field = query_field_field_get_db_field (value->field);
+
+		if (first_field)
+			first_field = FALSE;
+		else
+			g_string_append (string, ", ");
+
+		if (gda_value_is_null (value->value))
+			g_string_append_printf (string, "%s = NULL", field->name);
+		else {
+			gchar *str;
+			if (value->dh)
+				str = data_handler_get_sql_from_value (value->dh, value->value);
+			else
+				str = gda_value_stringify (value->value);
+			g_string_append_printf (string, "%s = %s", field->name, str);
+			g_free (str);
+		}
+
+		list = g_slist_next (list);
+	}
+
+	/* WHERE conditions */
+	str = make_where_cond_sql_part (form);
+	g_string_append (string, " ");
+	g_string_append (string, str);
+	g_free (str);
+
+	str = string->str;
+	g_string_free (string, FALSE);
+	return str;
+}
+
+static gchar *
+make_where_cond_sql_part (DataForm *form)
+{
+	gchar *str;
+	GString *string;
+	gboolean first_field = TRUE;
+	GSList *list;
+	DbTable *modif_table;
+
+
+	modif_table = query_exec_get_modif_table (form->priv->qx);
+	string = g_string_new ("WHERE ");
+
+	/* WHERE conditions from the currently selected row in the recordset */
+	list = query_exec_get_query_ref (form->priv->qx)->fields;
+	while (list) {
+		QueryField *qf;
+		qf = QUERY_FIELD (list->data);
+
+		if (qf->field_type == QUERY_FIELD_FIELD) {
+			DbField *field;
+			DbTable *table;
+			gint pos;
+			const GdaValue *value;
+
+			field = query_field_field_get_db_field (qf);
+			table = database_find_table_from_field (form->priv->qv->query->conf->db, field);
+			if (table == modif_table) { /* WHERE condition only on the modified table */
+				if (first_field)
+					first_field = FALSE;
+				else
+					g_string_append (string, "AND ");
+				
+				g_string_append_printf (string, "%s ", field->name);
+				pos = query_get_pos_by_field (query_exec_get_query_ref (form->priv->qx), qf);
+				value = server_resultset_get_gdavalue (query_exec_get_data_set (form->priv->qx),
+								       form->priv->numrow, pos);
+				if (gda_value_is_null (value))
+					g_string_append (string, "IS NULL ");
+				else {
+					gchar *str;
+					DataHandler *dh;
+					
+					dh = server_access_get_object_handler (form->priv->qv->query->conf->srv,
+									       G_OBJECT (field->type));
+					if (dh)
+						str = data_handler_get_sql_from_value (dh, value);
+					else
+						str = gda_value_stringify (value);
+					g_string_append_printf (string, "= %s ", str);
+					g_free (str);
+				}
+			}
+		}
+
+		list = g_slist_next (list);
+	}
+
+	/* WHERE conditions coming from the original Query */
+	/* FIXME: add the WHERE conditions coming from the original Query:
+	   keep only the QueryCond related to the modified table? */
+
+	str = string->str;
+	g_string_free (string, FALSE);
+	return str;
+}
+
+static gchar *
+make_insert_sql_query (DataForm *form, GSList *param_values)
+{
+	GString *string, *stringvals;
+	gchar *str;
+	DbTable *modif_table;
+	GSList *list;
+	gboolean first_field;
+
+	g_return_val_if_fail (form->priv->mode == DATA_FORM_INSERT_MODE, NULL);
+	string = g_string_new ("INSERT INTO ");
+	stringvals = g_string_new ("VALUES (");
+
+	modif_table = query_exec_get_modif_table (form->priv->qx);
+	g_string_append_printf (string, "%s (", modif_table->name);
+
+	/* Parsing the new values to create the insert query */
+	first_field = TRUE;
+	list = param_values;
+	while (list) {
+		QueryParamValue *value;
+		DbField *field;
+		
+		value = QUERY_PARAM_VALUE (list->data);
+		field = query_field_field_get_db_field (value->field);
+
+		if (first_field)
+			first_field = FALSE;
+		else {
+			g_string_append (string, ", ");
+			g_string_append (stringvals, ", ");
+		}
+
+		if (gda_value_is_null (value->value)) {
+			g_string_append_printf (string, "%s", field->name);
+			g_string_append (stringvals, "NULL");
+		}
+		else {
+			gchar *str;
+
+			g_string_append_printf (string, "%s", field->name);
+			if (value->dh)
+				str = data_handler_get_sql_from_value (value->dh, value->value);
+			else
+				str = gda_value_stringify (value->value);
+			g_string_append_printf (stringvals, "%s", str);
+			g_free (str);
+		}
+
+		list = g_slist_next (list);
+	}
+	g_string_append (string, ") ");
+	g_string_append (stringvals, ")");
+
+	g_string_append (string, stringvals->str);
+
+	str = string->str;
+	g_string_free (string, FALSE);
+	g_string_free (stringvals, TRUE);
+	return str;
+}
+
+
+static gchar * make_delete_sql_query (DataForm *form);
+
+/* delete the current selected entry */
+static void 
+form_do_delete (DataForm *form)
+{
+	gchar *query;
+	ServerResultset *rs;
+
+	/* build the SQL statement */
+	switch  (form->priv->mode) {
+	case DATA_FORM_EDIT_MODE:
+		query = make_delete_sql_query (form);
+		break;
+	default:
+		query = NULL;
+		g_warning ("form_do_delete () called on the wrong mode: %d\n", form->priv->mode);
+	}
+
+
+	rs = server_access_do_query (form->priv->qv->query->conf->srv, query, SERVER_ACCESS_QUERY_SQL);
+	if (rs) {
+		g_signal_emit (G_OBJECT (form), data_form_signals [ACTION_REFRESH], 0);
+		g_object_unref (G_OBJECT (rs));
+	}
+
+	g_free (query);
+}
+
+static gchar *
+make_delete_sql_query (DataForm *form)
+{
+	gchar *str;
+	GString *string;
+	DbTable *modif_table;
+
+	string = g_string_new ("DELETE FROM ");
+	modif_table = query_exec_get_modif_table (form->priv->qx);
+	g_string_append_printf (string, "%s ", modif_table->name);
+	
+	str = make_where_cond_sql_part (form);
+	g_string_append (string, str);
+	g_free (str);
+
+	str = string->str;
+	g_string_free (string, FALSE);
+	return str;
+}
+
+
+
+
+
+
+
+
 
 /*
  * Actions buttons
