@@ -791,7 +791,7 @@ database_load_tables (Database * db, ServerAccess * srv)
 			((DbItem *) table)->updated = TRUE;
 
 			g_signal_emit_by_name (G_OBJECT (db), "progress",
-					       _("Updating the list of tables..."),
+					       "TABLES",
 					       now, total);
 			now++;
 		}
@@ -958,7 +958,7 @@ database_load_views (Database * db, ServerAccess * srv)
 			((DbItem *) table)->updated = TRUE;
 
 			g_signal_emit_by_name (G_OBJECT (db), "progress",
-						 _("Updating the list of views..."),
+						 "VIEWS",
 						 now, total);
 			now++;
 		}
@@ -1085,7 +1085,7 @@ database_load_sequences (Database * db, ServerAccess * srv)
 			/* mark the sequence as updated */
 			DB_ITEM (seq)->updated = TRUE;
 			g_signal_emit_by_name (G_OBJECT (db), "progress",
-						 _("Updating the list of sequences..."),
+						 "SEQUENCES",
 						 now, total);
 			now++;
 		}
@@ -1394,7 +1394,7 @@ database_build_xml_tree (Database * db, xmlDocPtr doc)
 		/* table's fields */
 		list2 = table->fields;
 		while (list2) {
-			DataDisplayFns *fns;
+			DataHandler *dh;
 
 			field = DB_FIELD (list2->data);
 			subsubtree =
@@ -1423,16 +1423,16 @@ database_build_xml_tree (Database * db, xmlDocPtr doc)
 				xmlSetProp (subsubtree, "iskey", "t");
 			else
 				xmlSetProp (subsubtree, "iskey", "f");
-			if (field->default_val)
-				xmlSetProp (subsubtree, "default",
-					    field->default_val);
+			if (field->default_value) {
+				xmlNodePtr valnode;
+				valnode = gda_value_to_xml (field->default_value);
+				xmlAddChild (subsubtree, valnode);
+			}
 
 			/* if there is a plugin for that field, write it */
-			if ((fns =
-			     g_hash_table_lookup (db->srv->types_objects_hash,
-						  field))) {
-				xmlSetProp (subsubtree, "plugin",
-					    fns->plugin_name);
+			if ((dh = server_access_get_object_handler (db->srv, G_OBJECT (field)))) {
+				if (data_handler_is_plugin (dh))
+					xmlSetProp (subsubtree, "plugin", data_handler_get_plugin_name (dh));
 			}
 			list2 = g_slist_next (list2);
 		}
@@ -1782,7 +1782,9 @@ db_table_dispose (GObject   * object)
 	g_return_if_fail (IS_DB_TABLE (object));
 
 	table = DB_TABLE (object);
+#ifdef debug
 	g_print ("Disposing table %p (%s)\n", table, table->name);
+#endif
 
 	/* we do free the fields GSList and destroy every field */
 	list = table->fields;
@@ -1803,7 +1805,6 @@ static void
 db_table_finalize (GObject   * object)
 {
 	DbTable *table;
-	GSList *list;
 
 	g_return_if_fail (object != NULL);
 	g_return_if_fail (IS_DB_TABLE (object));
@@ -2196,26 +2197,28 @@ db_table_load_from_xml_node (Database * db, xmlNodePtr tabletree)
 			}
 			g_free (str);
 
-			str = xmlGetProp (node, "default");
-			if (str)
-				field->default_val = str;
+
+			if (node->xmlChildrenNode) {
+				xmlNodePtr valnode = node->xmlChildrenNode;
+
+				while (valnode) {
+					if (!strcmp (valnode->name, "value")) {
+						GdaValue *defval = gda_value_new_from_xml (valnode);
+						field->default_value = defval;
+					}
+					valnode = valnode->next;
+				}
+			}
 
 			/* plugin for this data type */
 			str = xmlGetProp (node, "plugin");
 			if (str) {
-				GSList *plist;
-				gboolean found = FALSE;
-				DataDisplayFns *fns;
-				plist = db->srv->data_types_display;
-				while (plist && !found) {
-					fns = (DataDisplayFns *) (plist->data);
-					if ((fns->plugin_name)
-					    && !strcmp (fns->plugin_name, str)) {
-						server_access_bind_object_display (db->srv, G_OBJECT (field), fns);
-						found = TRUE;
-					}
-					plist = g_slist_next (plist);
-				}
+				DataHandler *dh;
+				
+				dh = server_access_get_handler_from_name (db->srv, str);
+				if (dh)
+					server_access_bind_object_handler (db->srv, G_OBJECT (field), dh);
+				
 				g_free (str);
 			}
 		}
@@ -2282,7 +2285,7 @@ db_field_init (DbField * field)
 	field->length = 0;
 	field->null_allowed = TRUE;
 	field->is_key = FALSE;
-	field->default_val = NULL;
+	field->default_value = NULL;
 }
 
 
@@ -2310,9 +2313,9 @@ db_field_finalize (GObject   * object)
 		g_free (field->name);
 	field->name = NULL;
 	field->type = NULL;
-	if (field->default_val) {
-		g_free (field->default_val);
-		field->default_val = NULL;
+	if (field->default_value) {
+		gda_value_free (field->default_value);
+		field->default_value = NULL;
 	}
 
 	/* for the parent class */
@@ -2363,13 +2366,12 @@ db_field_load_contents (ServerAccess * srv, DbField * field,
 		field->is_key = FALSE;
 
 	value = gda_data_model_get_value_at (model, 8, row);
-	if (field->default_val) {
-		g_free (field->default_val);
-		field->default_val = NULL;
+	if (field->default_value) {
+		gda_value_free (field->default_value);
+		field->default_value = NULL;
 	}
 	if (value && !gda_value_is_null (value)) {
-		str = gda_value_stringify (value);
-		field->default_val = str;
+		field->default_value = gda_value_copy (value);
 	}
 }
 
@@ -2402,8 +2404,6 @@ void
 db_field_dump (DbField * field)
 {
 	/* prints all on one line */
-/*  gchar*       server_access_get_data_type (ServerAccess *srv, gchar *oid);*/
-
 	g_print ("%-20s %-10s\t%3d   ", field->name,
 		 field->type->sqlname,
 		 field->length);
@@ -2413,8 +2413,12 @@ db_field_dump (DbField * field)
 		g_print ("NOT NULL     ");
 	if (field->is_key)
 		g_print ("Key");
-	if (field->default_val)
-		g_print ("Defaults to %s", field->default_val);
+	if (field->default_value) {
+		gchar *str;
+		str = gda_value_stringify (field->default_value);
+		g_print ("Defaults to %s", str);
+		g_free (str);
+	}
 	g_print ("\n");
 }
 #endif
