@@ -19,6 +19,8 @@
  */
 
 #include <config.h>
+#include <libgnomedb/gnome-db-stock.h>
+#include <libgnomedb/gnome-db-util.h>
 #include "mainpagetable.h"
 #include "tableedit.h"
 #include "query.h"
@@ -106,8 +108,23 @@ main_page_table_init (MainPageTable * wid)
 	GtkWidget *sw, *bb;
 	gint i;
 
-	/* setting spaces,... */
-	gtk_container_set_border_width (GTK_CONTAINER (wid), GNOME_PAD / 2);
+	/* Toolbar */
+	bb = gnome_db_new_toolbar_widget (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
+	gtk_box_pack_start (GTK_BOX (wid), bb, FALSE, TRUE, GNOME_PAD / 2);
+	gtk_widget_show (bb);
+
+	wid->remove_table = gtk_toolbar_insert_stock (GTK_TOOLBAR (bb), GTK_STOCK_DELETE,
+						      _("Drop selected table"), NULL,
+						      GTK_SIGNAL_FUNC (remove_table_cb), wid, -1);
+	wid->edit_table = gtk_toolbar_insert_stock (GTK_TOOLBAR (bb), GTK_STOCK_PROPERTIES,
+						    _("View selected table properties"), NULL,
+						    GTK_SIGNAL_FUNC (edit_table_cb), wid, -1);
+	wid->view_table = gtk_toolbar_insert_stock (GTK_TOOLBAR (bb), GNOME_DB_STOCK_TABLES,
+						    _("View records for the selected table"), NULL,
+						    GTK_SIGNAL_FUNC (view_records_button_cb), wid, -1);
+	wid->new_table = gtk_toolbar_insert_stock (GTK_TOOLBAR (bb), GTK_STOCK_NEW,
+						   _("Create a new table"), NULL,
+						   NULL, NULL, -1); /* FIXME */
 
 	/* Scrolled Window for CList */
 	sw = gtk_scrolled_window_new (NULL, NULL);
@@ -137,39 +154,6 @@ main_page_table_init (MainPageTable * wid)
 	g_signal_connect (G_OBJECT (wid->clist), "button_press_event",
 			  G_CALLBACK (press_handler_event), wid);
 
-	/* Button Box */
-	bb = gtk_hbutton_box_new ();
-	gtk_button_box_set_layout (GTK_BUTTON_BOX (bb), GTK_BUTTONBOX_SPREAD);
-	gtk_box_pack_end (GTK_BOX (wid), bb, FALSE, TRUE, GNOME_PAD);
-	gtk_widget_show (bb);
-
-	/* Remove Table Button */
-	wid->remove_table = gtk_button_new_with_label (_("Drop Table"));
-	gtk_container_add (GTK_CONTAINER (bb), wid->remove_table);
-	gtk_widget_show (wid->remove_table);
-	g_signal_connect (G_OBJECT (wid->remove_table), "clicked",
-			  G_CALLBACK (remove_table_cb), wid);
-
-	/* Edit Table Button */
-	wid->edit_table = gtk_button_new_with_label (_("Properties"));
-	gtk_container_add (GTK_CONTAINER (bb), wid->edit_table);
-	gtk_widget_show (wid->edit_table);
-	g_signal_connect (G_OBJECT (wid->edit_table), "clicked",
-			  G_CALLBACK (edit_table_cb), wid);
-
-	/* View Table Button */
-	wid->view_table = gtk_button_new_with_label (_("View records"));
-	gtk_container_add (GTK_CONTAINER (bb), wid->view_table);
-	gtk_widget_show (wid->view_table);
-	g_signal_connect (G_OBJECT (wid->view_table), "clicked",
-			  G_CALLBACK (view_records_button_cb), wid);
-
-	/* New Table Button */
-	wid->new_table = gtk_button_new_with_label (_("Create Table"));
-	gtk_container_add (GTK_CONTAINER (bb), wid->new_table);
-	/* FIXME: use bonobo controls to create a table */
-	gtk_widget_show (wid->new_table);
-
 	wid->sel_row = -1;
 }
 
@@ -403,13 +387,16 @@ remove_table_dlg_response_cb (GtkDialog *dlg, gint reply, MainPageTable * wid)
 	switch (reply) {
 	case GTK_RESPONSE_YES:
 		if (rdata) {
+			ServerResultset *rs;
 			if (rdata->table->is_view)
 				query = g_strdup_printf ("drop view %s\n",
 							 rdata->table->name);
 			else
 				query = g_strdup_printf ("drop table %s\n",
 							 rdata->table->name);
-			server_access_do_query (rdata->conf->srv, query, SERVER_ACCESS_QUERY_SQL);
+			rs = server_access_do_query (rdata->conf->srv, query, SERVER_ACCESS_QUERY_SQL);
+			if (rs)
+				g_object_unref (G_OBJECT (rs));
 			g_free (query);
 			database_refresh (rdata->conf->db, rdata->conf->srv);
 		}
@@ -469,7 +456,6 @@ view_records_button_cb (GObject   * obj, MainPageTable * mpt)
 			GtkWidget *dlg;
 			if (!rdata->query) {	/* build the query here */
 				QueryField *qf;
-				QueryView *qv;
 				
 				if (rdata->table->is_view)
 					str = g_strdup_printf (_("contents of view %s"), rdata->table->name);
@@ -478,14 +464,12 @@ view_records_button_cb (GObject   * obj, MainPageTable * mpt)
 				rdata->query = QUERY (query_new (str, NULL, mpt->conf));
 				g_free (str);
 				
-				/* QueryView for the table */
-				qv = query_add_view_with_obj (rdata->query, G_OBJECT (rdata->table));
-				
 				/* QueryField */
+				qf = query_field_allfields_dbtable_new (rdata->query, rdata->table->name,
+									NULL, G_OBJECT (rdata->table));
 				qf = QUERY_FIELD (query_field_new (rdata->query, rdata->table->name, 
 								   QUERY_FIELD_ALLFIELDS));
 				
-				query_field_allfields_set_queryview (qf, qv);
 				query_field_set_is_printed (qf, TRUE);
 				query_add_field (QUERY (rdata->query), qf);
 				
@@ -502,12 +486,14 @@ view_records_button_cb (GObject   * obj, MainPageTable * mpt)
 			query_dump_contents (rdata->query);
 #endif
 			qx = QUERY_EXEC (query_exec_new (rdata->env));
-			dlg = query_exec_get_dialog_edit_form (qx, 0);
+			dlg = query_exec_get_edit_form (qx, 0);
 			if (dlg) {
 				rdata->exec_dlg = dlg;
 				gtk_widget_show (dlg);
 				g_object_add_weak_pointer (G_OBJECT (dlg), (gpointer) &(rdata->exec_dlg));
 			}
+			else
+				query_exec_free (QUERY_EXEC (qx));
 		}
 		else
 			gdk_window_raise (rdata->props_dlg->window);
