@@ -25,6 +25,7 @@
 #include <gtk/gtkobject.h>
 #include "conf-manager.h"
 #include "database.h"
+#include "data-handler.h"
 
 G_BEGIN_DECLS
 
@@ -77,8 +78,9 @@ typedef enum {
 typedef struct _QueryParamValue QueryParamValue;
 struct _QueryParamValue
 {
-	QueryField *field; /* referenced field for the missing value */
-	GdaValue   *value; /* using a GdaValue because a string can't handle bin data */
+	QueryField     *field; /* referenced field for the missing value */
+	GdaValue       *value; /* using a GdaValue because a string can't handle bin data */
+	DataHandler    *dh;    /* used to translate a GdaValue to a txt or SQL statement */
 };
 #define QUERY_PARAM_VALUE(st) ((QueryParamValue *) st)
 
@@ -145,6 +147,8 @@ struct _QueryClass
 	void (*query_view_added)          (Query * q,  QueryView *qv);
 	void (*query_view_removed)        (Query * q,  QueryView *qv);
 
+	void (*child_query_created)       (Query * q, Query *new_query);
+	void (*child_query_dropped)       (Query * q, Query *old_query);
 	void (*query_created)             (Query * q, Query *new_query);
 	void (*query_dropped)             (Query * q, Query *old_query);
 };
@@ -526,17 +530,22 @@ gchar     * query_field_get_xml_id        (QueryField *qf);
  */
 
 /* Representation of a table's of a view's field */
+/* qv == NULL => create a new QueryView if necessary */
+QueryField      * query_field_field_dbfield_new       (Query *q, gchar * name, 
+						       QueryView *qv, DbField *field);
 QueryFieldIface * query_field_field_get_iface         ();
 ServerDataType  * query_field_field_get_data_type     (QueryField *qf);
 /* qv can be NULL */
-void              query_field_field_set_field         (QueryField *qf, QueryView *qv, DbField *field);
 QueryView       * query_field_field_get_queryview     (QueryField *qf);
 DbField         * query_field_field_get_db_field      (QueryField *qf);
 
 
 /* Representation of all the fields of a table or a view, or a Query */
+QueryField      * query_field_allfields_dbtable_new   (Query *q, gchar * name, 
+						       QueryView *qv, DbTable *table);
+QueryField      * query_field_allfields_query_new     (Query *q, gchar * name, 
+						       QueryView *qv, Query *query);
 QueryFieldIface * query_field_allfields_get_iface     ();
-void              query_field_allfields_set_queryview (QueryField *qf, QueryView *qv);
 QueryView       * query_field_allfields_get_queryview (QueryField *qf);
 gint              query_field_allfields_get_span      (QueryField *qf);
 
