@@ -25,7 +25,8 @@
 #include <gmodule.h>
 #include <libgda/libgda.h>
 #include "marshal.h"
-
+#include "data-handler-default.h"
+#include "settings.h"
 
 /* signals */
 enum
@@ -33,21 +34,32 @@ enum
 	CONN_OPENED,
 	CONN_TO_CLOSE,
 	CONN_CLOSED,
+	DATA_TYPE_ADDED,
+	DATA_TYPE_REMOVED,
 	DATA_TYPES_UPDATED,
 	DATA_FUNCTION_ADDED,
 	DATA_FUNCTION_REMOVED,
-	DATA_FUNCTION_UPDATED,
+	DATA_FUNCTIONS_UPDATED,
 	DATA_AGGREGATE_ADDED,
 	DATA_AGGREGATE_REMOVED,
-	DATA_AGGREGATE_UPDATED,
+	DATA_AGGREGATES_UPDATED,
 	PROGRESS,
 	OBJECTS_BINDINGS_UPDATED,
 	LAST_SIGNAL
 };
 
-static gint server_access_signals[LAST_SIGNAL] = { 0, 0, 0, 0,
-	0, 0, 0, 0, 0, 0, 0
+typedef enum _ServerAccessOpMode ServerAccessOpMode;
+
+enum _ServerAccessOpMode
+{
+	SERVER_ACCESS_SELECT_OP,
+	SERVER_ACCESS_INSERT_OP,
+	SERVER_ACCESS_UPDATE_OP,
+	SERVER_ACCESS_DELETE_OP,
+	SERVER_ACCESS_UNKNOWN_OP
 };
+
+static gint server_access_signals[LAST_SIGNAL] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
 static void server_access_class_init (ServerAccessClass * class);
 static void server_access_init (ServerAccess * srv, ServerAccessClass *klass);
 static void server_access_dispose (GObject * object);
@@ -111,6 +123,22 @@ server_access_class_init (ServerAccessClass * class)
 			      NULL, NULL,
 			      marshal_VOID__VOID,
 			      G_TYPE_NONE, 0);
+	server_access_signals[DATA_TYPE_ADDED] =
+		g_signal_new ("data_type_added", 
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_FIRST,
+			      G_STRUCT_OFFSET (ServerAccessClass, data_type_added),
+			      NULL, NULL,
+			      marshal_VOID__POINTER,
+			      G_TYPE_NONE, 1, G_TYPE_POINTER);
+	server_access_signals[DATA_TYPE_REMOVED] =
+		g_signal_new ("data_type_removed", 
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_FIRST,
+			      G_STRUCT_OFFSET (ServerAccessClass, data_type_removed),
+			      NULL, NULL,
+			      marshal_VOID__POINTER,
+			      G_TYPE_NONE, 1, G_TYPE_POINTER);
 	server_access_signals[DATA_TYPES_UPDATED] =
 		g_signal_new ("data_types_updated", 
 			      G_TYPE_FROM_CLASS (object_class),
@@ -135,11 +163,11 @@ server_access_class_init (ServerAccessClass * class)
 			      NULL, NULL,
 			      marshal_VOID__POINTER,
 			      G_TYPE_NONE, 1, G_TYPE_POINTER);
-	server_access_signals[DATA_FUNCTION_UPDATED] =
-		g_signal_new ("data_function_updated", 
+	server_access_signals[DATA_FUNCTIONS_UPDATED] =
+		g_signal_new ("data_functions_updated", 
 			      G_TYPE_FROM_CLASS (object_class),
 			      G_SIGNAL_RUN_FIRST,
-			      G_STRUCT_OFFSET (ServerAccessClass, data_function_updated),
+			      G_STRUCT_OFFSET (ServerAccessClass, data_functions_updated),
 			      NULL, NULL,
 			      marshal_VOID__VOID,
 			      G_TYPE_NONE, 0);
@@ -159,11 +187,11 @@ server_access_class_init (ServerAccessClass * class)
 			      NULL, NULL,
 			      marshal_VOID__POINTER,
 			      G_TYPE_NONE, 1, G_TYPE_POINTER);
-	server_access_signals[DATA_AGGREGATE_UPDATED] =
-		g_signal_new ("data_aggregate_updated", 
+	server_access_signals[DATA_AGGREGATES_UPDATED] =
+		g_signal_new ("data_aggregates_updated", 
 			      G_TYPE_FROM_CLASS (object_class),
 			      G_SIGNAL_RUN_FIRST,
-			      G_STRUCT_OFFSET (ServerAccessClass, data_aggregate_updated),
+			      G_STRUCT_OFFSET (ServerAccessClass, data_aggregates_updated),
 			      NULL, NULL,
 			      marshal_VOID__VOID,
 			      G_TYPE_NONE, 0);
@@ -187,13 +215,15 @@ server_access_class_init (ServerAccessClass * class)
 	class->conn_opened = NULL;
 	class->conn_to_close = NULL;
 	class->conn_closed = m_conn_closed;
+	class->data_type_added = NULL;
+	class->data_type_removed = NULL;
 	class->data_types_updated = NULL;
 	class->data_function_added = NULL;
 	class->data_function_removed = NULL;
-	class->data_function_updated = NULL;
+	class->data_functions_updated = NULL;
 	class->data_aggregate_added = NULL;
 	class->data_aggregate_removed = NULL;
-	class->data_aggregate_updated = NULL;
+	class->data_aggregates_updated = NULL;
 	class->progress = NULL;
 	class->objects_bindings_updated = NULL;
 	object_class->dispose = server_access_dispose;
@@ -219,7 +249,7 @@ server_access_init (ServerAccess * srv, ServerAccessClass *klass)
 		g_warning ("GModule is required for GNOME-DB to run!");
 		exit (1);
 	}
-	srv->data_types_display = data_display_get_initial_list ();
+	srv->data_types_handlers = data_handlers_get_initial_list (srv);
 	srv->types_objects_hash = g_hash_table_new (NULL, NULL);	/* pointers! */
 	srv->weak_ref_objects = NULL;
 	srv->bindable_objects = NULL;
@@ -311,6 +341,18 @@ server_access_dispose (GObject * object)
 		srv->types_objects_hash = NULL;
 	}
 
+	/* Display functions and Plugins */
+	while (srv->data_types_handlers) {
+		GSList *list;
+		g_object_unref (G_OBJECT (srv->data_types_handlers->data));
+
+		list = srv->data_types_handlers;
+		srv->data_types_handlers = g_slist_remove_link (srv->data_types_handlers,
+								srv->data_types_handlers);
+		g_slist_free_1 (list);
+	}
+
+
 	/* chain to parent class */
 	parent_class->dispose (object);
 }
@@ -320,7 +362,6 @@ static void
 server_access_finalize (GObject * object)
 {
 	ServerAccess *srv;
-	GSList *list;
 
 	g_return_if_fail (object != NULL);
 	g_return_if_fail (IS_SERVER_ACCESS (object));
@@ -336,32 +377,6 @@ server_access_finalize (GObject * object)
 	g_string_free (srv->password, TRUE);
 	srv->password = NULL;
 
-	/* Display functions and Plugins */
-	while (srv->data_types_display) {
-		DataDisplayFns *fns;
-		fns = (DataDisplayFns *) (srv->data_types_display->data);
-		if (fns->plugin_file) {	/* it is a plugin */
-			void (*release_struct) (DataDisplayFns * fns);
-			g_free (fns->plugin_file);
-			g_module_symbol (fns->lib_handle, "release_struct", (gpointer *) &release_struct);
-			if (release_struct)
-				(release_struct) (fns);
-			g_module_close (fns->lib_handle);
-			if (release_struct)
-				fns = NULL;
-		}
-		else {		/* built in data type display */
-			data_display_free_display_fns (fns);
-			fns = NULL;
-		}
-		if (fns)
-			g_free (fns);
-		list = srv->data_types_display;
-		srv->data_types_display = g_slist_remove_link (srv->data_types_display,
-							       srv->data_types_display);
-		g_slist_free_1 (list);
-	}
-
 	/* chain to parent class */
 	parent_class->finalize (object);
 }
@@ -491,7 +506,7 @@ m_conn_closed (ServerAccess * srv)
 #ifdef debug_signal
 		g_print (">> 'DATA_TYPES_UPDATED' from m_conn_closed\n");
 #endif
-		g_signal_emit_by_name (G_OBJECT (srv), "data_types_updated");
+		g_signal_emit (G_OBJECT (srv), server_access_signals[DATA_TYPES_UPDATED], 0);
 #ifdef debug_signal
 		g_print ("<< 'DATA_TYPES_UPDATED' from m_conn_closed\n");
 #endif
@@ -499,9 +514,11 @@ m_conn_closed (ServerAccess * srv)
 	if (srv->data_functions) {
 		list = srv->data_functions;
 		while (list) {
-			g_object_unref (G_OBJECT (list->data));
 			hold = list;
 			list = g_slist_remove_link (list, list);
+			g_signal_emit (G_OBJECT (srv), server_access_signals[DATA_FUNCTION_REMOVED], 
+				       0, hold->data);
+			g_object_unref (G_OBJECT (hold->data));
 			g_slist_free_1 (hold);
 		}
 		srv->data_functions = NULL;
@@ -509,9 +526,11 @@ m_conn_closed (ServerAccess * srv)
 	if (srv->data_aggregates) {
 		list = srv->data_aggregates;
 		while (list) {
-			g_object_unref (G_OBJECT (list->data));
 			hold = list;
 			list = g_slist_remove_link (list, list);
+			g_signal_emit (G_OBJECT (srv), server_access_signals[DATA_AGGREGATE_REMOVED], 
+				       0, hold->data);
+			g_object_unref (G_OBJECT (hold->data));
 			g_slist_free_1 (hold);
 		}
 		srv->data_aggregates = NULL;
@@ -519,36 +538,121 @@ m_conn_closed (ServerAccess * srv)
 }
 
 
+static ServerAccessOpMode find_op_mode_sql (const gchar *query);
 ServerResultset *
 server_access_do_query (ServerAccess * srv, const gchar * query, ServerAccessQueryType type)
 {
 	ServerResultset *qres = NULL;
 
 	if (srv->cnc) {
-		GdaDataModel *res;
 		GdaCommandType gtype;
-		GdaCommand *cmd;
+		ServerAccessOpMode opmode = SERVER_ACCESS_UNKNOWN_OP;
+		gboolean ask_confirm = TRUE;
+		gboolean do_run = TRUE;
 
 		switch (type) {
 		case SERVER_ACCESS_QUERY_XML:
 			gtype = GDA_COMMAND_TYPE_XML;
+			/* FIXME: find comand type */
 			break;
 		default:
 			gtype = GDA_COMMAND_TYPE_SQL;
+			opmode = find_op_mode_sql (query);
 			break;
 		}
 
-		cmd = gda_command_new (query, gtype, GDA_COMMAND_OPTION_STOP_ON_ERRORS);
+		switch (opmode) {
+		case SERVER_ACCESS_SELECT_OP:
+			ask_confirm = settings_get_select_confirmation_dialog ();
+			break;
+		case SERVER_ACCESS_INSERT_OP:
+			ask_confirm = settings_get_insert_confirmation_dialog ();
+			break;
+		case SERVER_ACCESS_UPDATE_OP:
+			ask_confirm = settings_get_update_confirmation_dialog ();
+			break;
+		case SERVER_ACCESS_DELETE_OP:
+			ask_confirm = settings_get_delete_confirmation_dialog ();
+			break;
+		default:
+			break;
+		}
 
-		res = gda_connection_execute_single_command (srv->cnc, cmd, NULL);
+		if (ask_confirm) {
+			GtkWidget *dialog, *label, *hb, *pixmap, *vb;
+			gchar *str;
+			gint answer;
 
-		if (res)
-			qres = SERVER_RESULTSET (server_resultset_new (res, cmd));
+			dialog = gtk_dialog_new_with_buttons (NULL, NULL, 0,
+							      GTK_STOCK_YES, GTK_RESPONSE_YES,
+							      GTK_STOCK_NO, GTK_RESPONSE_NO, NULL);
+
+			str = g_strdup_printf (_("<span weight=\"bold\" size=\"larger\">Run this "
+						 "query?</span>\n\n%s"), query);
+			label = gtk_label_new (NULL);
+			gtk_label_set_markup (GTK_LABEL (label), str);
+			g_free (str);
+
+			hb = gtk_hbox_new (FALSE, GNOME_PAD);
+			gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), hb, TRUE, TRUE, GNOME_PAD);
+			gtk_widget_show (hb);
+
+			vb = gtk_vbox_new (FALSE, 0);
+			gtk_box_pack_start (GTK_BOX (hb), vb, FALSE, FALSE, GNOME_PAD/2.);
+			gtk_widget_show (vb);
+			
+			pixmap = gtk_image_new_from_stock (GTK_STOCK_DIALOG_QUESTION, GTK_ICON_SIZE_DIALOG);
+			gtk_box_pack_start (GTK_BOX (vb), pixmap, FALSE, FALSE, GNOME_PAD/2.);
+			gtk_widget_show (pixmap);
+
+			vb = gtk_vbox_new (FALSE, 0);
+			gtk_box_pack_start (GTK_BOX (hb), vb, TRUE, TRUE, GNOME_PAD/2.);
+			gtk_widget_show (vb);
+			
+			gtk_box_pack_start (GTK_BOX (vb), label, TRUE, TRUE, GNOME_PAD/2.);
+			gtk_widget_show (label);
+			answer = gtk_dialog_run (GTK_DIALOG (dialog));
+			gtk_widget_destroy (dialog);
+			if (answer == GTK_RESPONSE_NO) 
+				do_run = FALSE;
+
+		}
+		
+		if (do_run) {
+			GdaDataModel *res;
+			GdaCommand *cmd;
+
+			cmd = gda_command_new (query, gtype, GDA_COMMAND_OPTION_STOP_ON_ERRORS);
+#ifdef debug
+			g_print ("ServerAccess Run: %s\n", query);
+#endif
+			res = gda_connection_execute_single_command (srv->cnc, cmd, NULL);
+			
+			if (res)
+				qres = SERVER_RESULTSET (server_resultset_new (res, cmd));
+		}
 	}
 
 	return qres;
 }
 
+static ServerAccessOpMode
+find_op_mode_sql (const gchar *query)
+{
+	ServerAccessOpMode mode = SERVER_ACCESS_UNKNOWN_OP;
+
+	if (! g_ascii_strncasecmp (query, "SELECT", 6))
+		mode = SERVER_ACCESS_SELECT_OP;
+	if (! g_ascii_strncasecmp (query, "INSERT", 6))
+		mode = SERVER_ACCESS_INSERT_OP;
+	if (! g_ascii_strncasecmp (query, "UPDATE", 6))
+		mode = SERVER_ACCESS_UPDATE_OP;
+	if (! g_ascii_strncasecmp (query, "DELETE", 6))
+		mode = SERVER_ACCESS_DELETE_OP;
+
+	return mode;
+}
+
 
 /*
  * data type, etc lookup and management
@@ -592,7 +696,7 @@ server_access_build_xml_tree (ServerAccess * srv, xmlDocPtr doc)
 	list = srv->data_types;
 	while (list) {
 		ServerDataType *dt;
-		DataDisplayFns *fns;
+		DataHandler *dh;
 
 		dt = SERVER_DATA_TYPE (list->data);
 		subtree = xmlNewChild (tree, NULL, "type", NULL);
@@ -607,8 +711,8 @@ server_access_build_xml_tree (ServerAccess * srv, xmlDocPtr doc)
 		xmlSetProp (subtree, "gdatype", str);
 		g_free (str);
 		/* if there is a plugin for that data type, write it */
-		if ((fns = g_hash_table_lookup (srv->types_objects_hash, dt))) {
-			xmlSetProp (subtree, "plugin", fns->plugin_name);
+		if ((dh = g_hash_table_lookup (srv->types_objects_hash, dt))) {
+			xmlSetProp (subtree, "plugin", data_handler_get_plugin_name (dh));
 		}
 		list = g_slist_next (list);
 	}
@@ -720,10 +824,8 @@ server_access_build_from_xml_tree (ServerAccess * srv, xmlNodePtr node)
 					g_free (str);
 
 					/* default display function */
-					dt->display_fns =
-						server_access_get_display_fns_from_gda (srv, dt);
-					srv->data_types =
-						g_slist_append (srv->data_types, dt);
+					dt->data_handler = server_access_get_handler_from_type (srv, dt);
+					srv->data_types = g_slist_append (srv->data_types, dt);
 
 					/* plugin for this data type */
 					if ((str =
@@ -731,16 +833,16 @@ server_access_build_from_xml_tree (ServerAccess * srv, xmlNodePtr node)
 							 "plugin"))) {
 						GSList *plist;
 						gboolean found = FALSE;
-						DataDisplayFns *fns;
-						plist = srv->
-							data_types_display;
+						DataHandler *dh;
+						plist = srv->data_types_handlers;
 						while (plist && !found) {
-							fns = (DataDisplayFns *) (plist->data);
-							if ((fns->plugin_name)
-							    && !strcmp (fns->plugin_name, str))
+							dh = DATA_HANDLER (plist->data);
+							if ((data_handler_get_plugin_name (dh)) &&
+							    !strcmp (data_handler_get_plugin_name (dh), str))
 							{
-								server_access_bind_object_display (srv, G_OBJECT (dt),
-												   fns);
+								server_access_bind_object_handler (srv, 
+												   G_OBJECT (dt),
+												   dh);
 								found = TRUE;
 							}
 							plist = g_slist_next (plist);
@@ -884,35 +986,67 @@ server_access_build_from_xml_tree (ServerAccess * srv, xmlNodePtr node)
 	return ok;
 }
 
-DataDisplayFns *
-server_access_get_display_fns_from_gda (ServerAccess * srv, ServerDataType * dt)
+DataHandler *
+server_access_get_handler_from_name (ServerAccess * srv, const gchar *plugin_name)
 {
-	DataDisplayFns *fns = NULL, *ptr;
+	GSList *list;
+	DataHandler *dh, *retval = NULL;
+
+	g_return_val_if_fail (srv && IS_SERVER_ACCESS (srv), NULL);
+
+	list = srv->data_types_handlers;
+	while (list && !retval) {
+		dh = DATA_HANDLER (list->data);
+		if (data_handler_is_plugin (dh) &&
+		    !strcmp (data_handler_get_plugin_name (dh), plugin_name)) {
+			retval = dh;
+		}
+		list = g_slist_next (list);
+	}
+
+	return retval;
+}
+
+DataHandler *
+server_access_get_handler_from_gda (ServerAccess * srv, GdaValueType gda_type)
+{
+	DataHandler *dh = NULL, *ptr;
 	gint i;
 	GSList *list;
 
-	list = srv->data_types_display;	/* SHOULD NEVER BE NULL! */
-	while (list && !fns) {
-		ptr = (DataDisplayFns *) (list->data);
-		if (ptr->nb_gda_type == 0)	/* 0 => ALL gda types */
-			fns = ptr;
+	g_return_val_if_fail (srv && IS_SERVER_ACCESS (srv), NULL);
+
+	list = srv->data_types_handlers;	/* SHOULD NEVER BE NULL! */
+	while (list && !dh) {
+		ptr = DATA_HANDLER (list->data);
+		if (data_handler_get_nb_gda_type (ptr) == 0)	/* 0 => ALL gda types */
+			dh = ptr;
 		i = 0;
-		while (!fns && (i < ptr->nb_gda_type)) {
-			if (ptr->valid_gda_types[i] == dt->gda_type)
-				fns = ptr;
+		while (!dh && (i < data_handler_get_nb_gda_type (ptr))) {
+			if (data_handler_get_gda_type (ptr, i) == gda_type)
+				dh = ptr;
 			i++;
 		}
 
 		list = g_slist_next (list);
 	}
 
-	return fns;
+	return dh;
 }
 
-DataDisplayFns *
-server_access_get_object_display_fns (ServerAccess * srv, GObject * obj)
+DataHandler *
+server_access_get_handler_from_type (ServerAccess * srv, ServerDataType * dt)
 {
-	DataDisplayFns *fns = NULL;
+	g_return_val_if_fail (srv && IS_SERVER_ACCESS (srv), NULL);
+	g_return_val_if_fail (dt && IS_SERVER_DATA_TYPE (dt), NULL);
+
+	return server_access_get_handler_from_gda (srv, dt->gda_type);
+}
+
+DataHandler *
+server_access_get_object_handler (ServerAccess * srv, GObject * obj)
+{
+	DataHandler *dh = NULL;
 	GSList *list;
 	gpointer data;
 
@@ -920,43 +1054,44 @@ server_access_get_object_display_fns (ServerAccess * srv, GObject * obj)
 
 	if (obj && G_IS_OBJECT (obj)) {
 		/* check to see if there is a specified bind for that object */
-		fns = g_hash_table_lookup (srv->types_objects_hash, obj);
-
-		/* if no fns has been found, we get the default ones */
-		if (!fns && IS_SERVER_DATA_TYPE (obj))
-			fns = SERVER_DATA_TYPE (obj)->display_fns;
+		dh = g_hash_table_lookup (srv->types_objects_hash, obj);
+		
+		/* if no dh has been found, we get the default ones */
+		if (!dh && IS_SERVER_DATA_TYPE (obj))
+			dh = SERVER_DATA_TYPE (obj)->data_handler;
 
 		/* if no function found, then let's try to find a plugin for the 
 		   object, using the declared functions to bind objects */
 		list = srv->bindable_objects;
-		while (list && !fns) {
+		while (list && !dh) {
 			data = ((gpointer (*)(GObject *)) (list->data))
 				(obj);
 			if (data) {
-				fns = g_hash_table_lookup (srv->types_objects_hash, data);
+				dh = g_hash_table_lookup (srv->types_objects_hash, data);
 
-				/* if no fns has been found, we get the default ones */
-				if (!fns && G_IS_OBJECT (data)
+				/* if no dh has been found, we get the default ones */
+				if (!dh && G_IS_OBJECT (data)
 				    && IS_SERVER_DATA_TYPE (data)) {
-					fns = SERVER_DATA_TYPE (data)->display_fns;
+					dh = SERVER_DATA_TYPE (data)->data_handler;
 				}
 			}
 			list = g_slist_next (list);
 		}
 	}
-	if (!fns)		/* default basic functions for every object */
-		fns = (DataDisplayFns *) srv->data_types_display->data;
-	return fns;
+
+	if (!dh) /* default basic functions for every object */
+		dh = DATA_HANDLER (srv->data_types_handlers->data);
+	return dh;
 }
 
 void
-server_access_bind_object_display (ServerAccess * srv,
-				   GObject * obj, DataDisplayFns * fns)
+server_access_bind_object_handler (ServerAccess * srv, GObject * obj, DataHandler *dh)
 {
 	g_return_if_fail (srv && IS_SERVER_ACCESS (srv));
 	g_return_if_fail (obj && G_IS_OBJECT (obj));
+	g_return_if_fail (dh && IS_DATA_HANDLER (dh));
 
-	g_hash_table_insert (srv->types_objects_hash, obj, fns);
+	g_hash_table_insert (srv->types_objects_hash, obj, dh);
 	g_object_weak_ref (G_OBJECT (obj), 
 			   (GWeakNotify) bound_object_destroy_cb, srv);
 	srv->weak_ref_objects = g_slist_append (srv->weak_ref_objects, obj);
@@ -966,24 +1101,24 @@ server_access_bind_object_display (ServerAccess * srv,
 gboolean 
 server_access_is_object_bound (ServerAccess * srv, GObject * obj)
 {
-	DataDisplayFns *fns;
+	DataHandler *dh;
 
 	g_return_val_if_fail (srv && IS_SERVER_ACCESS (srv), FALSE);
-	fns = g_hash_table_lookup (srv->types_objects_hash, obj);
+	dh = g_hash_table_lookup (srv->types_objects_hash, obj);
 
-	return fns ? TRUE : FALSE;
+	return dh ? TRUE : FALSE;
 }
 
 static void
 bound_object_destroy_cb (ServerAccess * srv, GObject * obj)
 {
 	srv->weak_ref_objects = g_slist_remove (srv->weak_ref_objects, obj);
-	server_access_unbind_object_display (srv, obj);
+	server_access_unbind_object_handler (srv, obj);
 }
 
 
 void
-server_access_unbind_object_display (ServerAccess * srv, GObject * obj)
+server_access_unbind_object_handler (ServerAccess * srv, GObject * obj)
 {
 	g_return_if_fail (srv && IS_SERVER_ACCESS (srv));
 	g_return_if_fail (obj && G_IS_OBJECT (obj));
@@ -1031,32 +1166,20 @@ server_access_rescan_display_plugins (ServerAccess * srv, gchar * path)
 		dir = readdir (dstream);
 		while (dir) {
 			if ((strlen (dir->d_name) > 3) &&	/* only .so files */
-			    !strcmp (dir->d_name + strlen (dir->d_name) - 3,
-				     ".so")) {
+			    !strcmp (dir->d_name + strlen (dir->d_name) - 3, ".so")) {
 				GModule *lib;
-				DataDisplayFns *fns;
-				DataDisplayFns *(*fpi) ();
+				DataHandler *dh;
+				DataHandler *(*fpi) (ServerAccess *srv);
 
-				str = g_strdup_printf ("%s/%s", path,
-						       dir->d_name);
+				str = g_strdup_printf ("%s/%s", path, dir->d_name);
 				if ((lib = g_module_open (str, G_MODULE_BIND_LAZY))) {
-				        g_module_symbol (lib,
-							"fetch_plugin_interface",
-							 (gpointer *) &fpi);
+				        g_module_symbol (lib, "fetch_plugin_interface", (gpointer *) &fpi);
 					if (fpi) {
-						fns = (fpi) ();
-						g_print ("\tNew plugin (%p/%p): %s(V%s)\n", fns, 
-							 fns->widget_from_value, fns->plugin_name, fns->version);
-						fns->lib_handle = lib;
-						fns->plugin_file =
-							g_strdup (str);
-						srv->data_types_display =
-							g_slist_append (srv->
-									data_types_display,
-									fns);
-						newones =
-							g_slist_append
-							(newones, fns);
+						dh = (fpi) (srv);
+						data_handler_init_plugin (dh, lib, str);
+						srv->data_types_handlers = 
+							g_slist_append (srv->data_types_handlers, dh);
+						newones = g_slist_append (newones, dh);
 					}
 					else {
 						g_module_close (lib);
@@ -1074,26 +1197,25 @@ server_access_rescan_display_plugins (ServerAccess * srv, gchar * path)
 		 */
 		fhs = g_new (struct foreach_hash_struct, 1);
 		fhs->hash = srv->types_objects_hash;
-		list = srv->data_types_display;
+		list = srv->data_types_handlers;
 		while (list) {
-			if ((((DataDisplayFns *) (list->data))->plugin_name) && !g_slist_find (newones, list->data)) {
+			if (data_handler_get_plugin_name (DATA_HANDLER (list->data)) && 
+			    !g_slist_find (newones, list->data)) {
+
 				/* only old plugins */
-				void (*release_struct) (DataDisplayFns *fns);
-				DataDisplayFns *fns;
+				DataHandler *dh;
 				gboolean found;
 
-				fns = (DataDisplayFns *) (list->data);
-				fhs->rem_value = fns;
+				dh = DATA_HANDLER (list->data);
+				fhs->rem_value = dh;
 				fhs->repl_value = NULL;
 
 				/* is there a new plugin with the same key? */
-				key = (fns->get_unique_key) ();
+				key = data_handler_get_key (dh);
 				list2 = newones;
 				found = FALSE;
 				while (list2 && !found) {
-					key2 = (((DataDisplayFns *) (list2->data))->
-						get_unique_key)
-						();
+					key2 = data_handler_get_key (DATA_HANDLER (list2->data));
 					if (!strcmp (key, key2)) {
 						fhs->repl_value = list2->data;
 						found = TRUE;
@@ -1105,19 +1227,13 @@ server_access_rescan_display_plugins (ServerAccess * srv, gchar * path)
 
 				g_hash_table_foreach (srv->types_objects_hash,
 						      (GHFunc) (foreach_hash_cb), fhs);
-				srv->data_types_display = g_slist_remove_link (srv->data_types_display,
+				srv->data_types_handlers = g_slist_remove_link (srv->data_types_handlers,
 									       list);
 				g_slist_free_1 (list);
 				/* freeing the ressources for that plugin */
-				g_free (fns->plugin_file);
-				g_module_symbol (fns->lib_handle,
-						 "release_struct",
-						 (gpointer *) &release_struct);
-				if (release_struct)
-					(release_struct) (fns);
-				g_module_close (fns->lib_handle);
+				g_object_unref (G_OBJECT (dh));
 
-				list = srv->data_types_display;
+				list = srv->data_types_handlers;
 			}
 			else
 				list = g_slist_next (list);
